// The MIT License (MIT)
//
// Copyright (c) 2018 Mateusz Pusz
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#pragma once

#include "hacks.h"
#include <type_traits>

namespace std::experimental::units {

  template<typename BaseType>
  struct downcast_base {
    using base_type = BaseType;
  };

  template<typename T>
  concept bool Downcastable =
      requires {
        typename T::base_type;
      } &&
      std::derived_from<T, downcast_base<typename T::base_type>>;

  template<Downcastable T>
  using downcast_from = T::base_type;

  template<Downcastable T>
  using downcast_to = std::type_identity<T>;

  template<Downcastable T>
  struct downcasting_traits : downcast_to<T> {};

  template<Downcastable T>
  using downcasting_traits_t = downcasting_traits<T>::type;

}  // namespace std::experimental::units
